#!/bin/sh
NET_INTF="eth0"
readonly NET_UCI_PROTO="network.${NET_INTF}.proto"
readonly NET_HTNM="/proc/sys/kernel/hostname"
_net_conf_host() {
	NET_INTF="wlp3s0"
	return 0
}
_net_add_ip() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local rslt
	local ip
	local mask
	utils_cmd_exists "ip" || return $?
	utils_cmd_exists "grep" || return $?
	rslt=$(ip addr show "${NET_INTF}" 2>/dev/null | \
		grep -Eo "([0-9]+\.){3}[0-9]+/[0-9]*" 2>/dev/null) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' getting IP address!"
		return ${ret}
	elif [ -z "${rslt}" ]; then
		jlog_err ${f} "Could not get IP address!"
		return 1
	fi
	IFS='/' read ip mask <<< "${rslt}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' separating IP and Mask!"
		return ${ret}
	fi
	json_add_array "ipv4-address"
	json_add_string "address" "${ip}"
	json_add_int "mask" ${mask}
	json_close_array
	return 0
}
_net_add_proto() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local uci
	local proto
	utils_uci_cmd uci || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not get UCI tool's command line!"
		return ${ret}
	fi
	proto=$(${uci} get "${NET_UCI_PROTO}" 2>&1) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error in UCI command!"
		jlog_err ${f} "${proto}"
		return ${ret}
	fi
	json_add_string "proto" "${proto}"
	return 0
}
_net_add_dns() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local line=""
	local token=""
	local dns=""
	json_add_array "dns-server"
	while read -r line; do
		read -r token dns <<< "${line}" || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Error '${ret}' getting DNS IP Address!"
			continue
		fi
		[ "${token}" = "nameserver" ] && json_add_string "" "${dns}"
	done < "/etc/resolv.conf"
	json_close_array
	return 0
}
_net_add_route() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local route
	utils_cmd_exists "ip" || return $?
	utils_cmd_exists "grep" || return $?
	utils_cmd_exists "awk" || return $?
	route=$(ip route list 2>/dev/null | grep -w "default" 2>/dev/null | \
		awk '{print $3}' 2>/dev/null) || ret=$?
        if [ -z "$route" ]; then
           route = "0.0.0.0"
        fi
	#	jlog_err ${f} "Error '${ret}' getting Gateway IP address!"
	#	jlog_err ${f} "Could not get Gateway IP address!"
	json_add_array "route"
	json_add_object
	json_add_string "target" "0.0.0.0"
	json_add_string "mask" "0"
	json_add_string "nexthop" "${route}"
	json_add_string "source" "0.0.0.0/0"
	json_close_object
	json_close_array
	return 0
}
_net_status() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local json
	utils_check_retv "${retv}" "json" || return $?
	json_init
	_net_add_ip || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error getting 'ip' and 'mask' from system!"
		return ${ret}
	fi
	_net_add_proto || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error getting 'proto' status!"
		return ${ret}
	fi
	_net_add_dns || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error getting 'dns' servers!"
		return ${ret}
	fi
	_net_add_route || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error getting 'route' gateway!"
		return ${ret}
	fi
	json=$(json_dump)
	json_cleanup
	eval "${retv}=\${json}"
	return 0
}
_net_hostname_set() {
	local f="${FUNCNAME[0]}"
	local ret=0
	local htnm
	json_select "${UBUS_SIGN_IDX}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Could not select signature!"
		return ${ret}
	fi
	json_get_var htnm "hostname"
	if [ -z "${htnm}" ]; then
		jlog_err ${f} "Cannot set an empty hostname!"
		return 1
	fi
	echo "${htnm}" > ${NET_HTNM} 2>/dev/null || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' setting hostname!"
		return ${ret}
	fi
	slog_inf ${f} "Hostname changed to '${htnm}'"
	return 0
}
_net_hostname_get() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local hostname
	local json
	utils_check_retv "${retv}" "hostname" "json" || return $?
	read -r hostname < "${NET_HTNM}" || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' in 'read' command!"
		return ${ret}
	fi
	json='{"hostname":"'${hostname}'"}'
	eval "${retv}=\${json}"
	return 0
}
_net_ip() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local retc=0
	local rslt
	local ip
	local json
	utils_check_retv "${retv}" "rslt" "ip" "json" || return $?
	utils_cmd_exists "ip" || return $?
	utils_cmd_exists "grep" || return $?
	rslt=$(ip addr show "${NET_INTF}" | grep -Eo "([0-9]+\.){3}[0-9]+/[0-9]*")
	IFS='/' read ip dummy <<< "${rslt}"
	json='{"ip":"'"${ip}"'"}'
	eval "${retv}=\${json}"
	return 0
}
do_net() {
	local proc="${1}"
	local retv="${2}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local nres=""
	utils_check_retv "${retv}" "proc" "nres" || return $?
	if [ -z "${proc}" ] || [ -z "${retv}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	# TEST MODE! When testing the Backend Shell Service handler for Network
	if [ ${BE_TGT} -eq 0 ]; then
		_net_conf_host || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not configure host for testing!"
			return ${ret}
		fi
	fi
	case "${proc}" in
	"hostname")
		_net_hostname_get nres || ret=$?
		;;
	"hostname_set")
		_net_hostname_set || ret=$?
		;;
	"ip")
		_net_ip nres || ret=$?
		;;
	"status")
		_net_status nres || ret=$?
		;;
	*)
		jlog_err ${f} "Procedure '${proc}' is invalid!"
		ret=1
		;;
	esac
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error occurred in '${proc}' procedure!"
		return ${ret}
	fi
	eval "${retv}=\${nres}"
	return 0
}
INC_NET=1
