#!/bin/sh
. ${SH}/be_utils_app.sh
. ${SH}/be_utils_jq.sh
readonly MKTEMP_TMPLT="webui-be.XXXXXX"
utils_cmd_exists() {
	local cmd="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	if [ -z "${cmd}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	command -v "${cmd}" &>/dev/null || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Command '"${cmd}"' not found!"
	fi
	return ${ret}
}
utils_uci_cmd() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local cmd=""
	utils_check_retv "${retv}" "cmd" || return $?
	utils_cmd_exists "uci" || return $?
	if [ -z "${retv}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	[ ${BE_TGT} -eq 0 ] && cmd=" -c ${BE_UCI}"
	cmd="uci${cmd}"
	eval "${retv}=\${cmd}"
	return 0
}
utils_rm_file() {
	local file="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	utils_cmd_exists "rm" || return $?
	if [ -z "${file}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	rm -f "${file}" >/dev/null 2>&1 || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error '${ret}' removing '${file}'!"
		return ${ret}
	fi
	return 0
}
utils_mktemp_file() {
	local retv="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local mkt_rslt
	utils_check_retv "${retv}" "mkt_rslt" || return $?
	utils_cmd_exists "mktemp" || return $?
	if [ -z "${retv}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	mkt_rslt=$(mktemp -t "${MKTEMP_TMPLT}" 2>&1) || ret=$?
	if [ ${ret} -ne 0 ]; then
		jlog_err ${f} "Error creating temporary file!"
		jlog_err ${f} "mktemp: ${mkt_rslt}"
		return ${ret}
	fi
	eval "${retv}=\${mkt_rslt}"
	return 0
}
utils_json_load() {
	local json="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local tmp_file="/tmp/${f}.${RANDOM}"
	local rslt
	utils_cmd_exists "cat" || return $?
	if [ -z "${json}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	if [ -e "${tmp_file}" ]; then
		utils_rm_file "${tmp_file}" || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not remove the temporary file!"
			return ${ret}
		fi
	fi
	json_init
	json_load "${json}" >"${tmp_file}" 2>&1
	[ -s "${tmp_file}" ] && ret=2
	if [ ${ret} -ne 0 ]; then
		jlog_err "${f}" "Attempted to load an invalid JSON object!"
		jlog_err "${f}" "json_load: $(cat ${tmp_file})"
		utils_rm_file "${tmp_file}"
		return ${ret}
	fi
	if [ -e "${tmp_file}" ]; then
		utils_rm_file "${tmp_file}" || ret=$?
		if [ ${ret} -ne 0 ]; then
			jlog_err ${f} "Could not remove the temporary file!"
			return ${ret}
		fi
	fi
	return 0
}
utils_rem_squote() {
	local str="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local rslt
	utils_cmd_exists "sed" || return $?
	if [ -z "${str}" ]; then
		slog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	rslt=$(sed -e "s/^'//" -e "s/'$//" 2>&1 <<< "${str}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		slog_err ${f} "Error in 'sed' command!"
		slog_err ${f} "${rslt}"
		return ${ret}
	fi
	echo ${rslt}
	return 0
}
utils_rem_dquote() {
	local str="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local rslt
	utils_cmd_exists "sed" || return $?
	if [ -z "${str}" ]; then
		slog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	rslt=$(sed -e 's/^"//' -e 's/"$//' 2>&1 <<< "${str}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		slog_err ${f} "Error in 'sed' command!"
		slog_err ${f} "${rslt}"
		return ${ret}
	fi
	echo ${rslt}
	return 0
}
utils_check_retv() {
	local nm_ret="${1}" && shift
	local nm_loc="${@}"
	local f="${FUNCNAME[0]}"
	if [ -z "${nm_ret}" ] || [ -z "${nm_loc}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	for nm in ${nm_loc}; do
		if [ "${nm_ret}" = "${nm}" ]; then
			jlog_err ${f} "Local variable '${nm}' collides with return variable!"
			return 2
		fi
	done
	return 0
}
utils_is_num() {
	local val="${1}"
	local f="${FUNCNAME[0]}"
	if [ -z "${val}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 2
	fi
	case ${val#[-+]} in
	*[!0-9]*|'')
		return 1
		;;
	*)
		return 0
		;;
	esac
	jlog_err ${f} "Should not reach this point!"
	return 3
}
utils_ip_valid() {
	local ip="${1}"
	local f="${FUNCNAME[0]}"
	local ret=0
	local rslt
	utils_cmd_exists "grep" || return $?
	if [ -z "${ip}" ]; then
		jlog_err ${f} "Missing mandatory arguments!"
		return 1
	fi
	rslt=$(grep -v ":" 2>&1 <<< "${ip}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		if [ -z "${rslt}" ]; then
			return 0
		fi
		jlog_err ${f} "Error in 'grep' command!"
		jlog_err ${f} "${rslt}"
		return ${ret}
	fi
	rslt=$(grep -Eo "([0-9]+\.){3}[0-9]+" 2>&1 <<< "${ip}") || ret=$?
	if [ ${ret} -ne 0 ]; then
		if [ -z "${rslt}" ]; then
			jlog_err ${f} "IP address '${ip}' is invalid!"
			return 2
		fi
		jlog_err ${f} "Error in 'grep' command!"
		jlog_err ${f} "${rslt}"
		return ${ret}
	fi
	if [ "${rslt}" != "${ip}" ]; then
		jlog_err ${f} "Input '${ip}' does not match parsed '${rslt}'!"
		return 3
	fi
	return 0
}
utils_parent_dir() {
	local dir="${1}"
	local retv="${2}"
	local f="${FUNCNAME[0]}"
	local odir
	local base
	utils_check_retv "${retv}" "dir" "odir" "base" || return $?
	if [ -z "${dir}" ]; then
		jlog_err "${f}" "Missing mandatory arguments!"
		return 1
	fi
	odir="${dir%/}"
	while [ "${odir}" != "${odir%/}" ]; do
		odir="${odir%/}"
	done
	# root directory. Put back one slash and proceed!
	[ -z "${odir}" ] && odir="/"
	base="${odir##*/}"
	# Remove the last component from the directory. Gets the parent!
	odir="${odir%${base}}"
	eval "${retv}=\${odir}"
	return 0
}
INC_UTILS=1
